from django import template
from django.core.urlresolvers import reverse

from ...core.defaults import (
    DEFAULT_ERROR_MESSAGE, DEFAULT_LOGIN_LABEL, DEFAULT_SIGNIN_LABEL)


register = template.Library()


@register.simple_tag(takes_context=True)
def github_login(context, label=None, css_class=None, icon_class=None,
                 only_login=False, error_message=None, error_class=None):
    """
    Renders a 'Sign in with GitHub' button.

    Parameters:
        - label: text to show as button label.
        - css_class: css class to apply to the button.
        - icon_class: css class to apply to the empty <i> tag placed inside
            (left) of the button, useful if used with css icons like
            "Font Awesome" or "Glyphicons".
        - only_login: if True the button will only try to log the user in
            without registering it, if an user that matches the GitHub
            credentials is not found then the user will be redirected to the
            page where the button was originally displayed and an a error
            message will be shown.
        - error_message: text to show as login error, if applicable.
        - error_class: css class to render the error message, if applicable.
    """
    replacements = {
        'service': 'GitHub'
    }

    context['action'] = reverse('socialnetworks:github:login')
    context['css_class'] = css_class
    context['icon_class'] = icon_class
    context['error_class'] = error_class
    context['only_login'] = only_login

    context['label'] = (
        label or
        (DEFAULT_LOGIN_LABEL if only_login else DEFAULT_SIGNIN_LABEL) %
        replacements
    )
    context['error_message'] = (
        error_message or
        DEFAULT_ERROR_MESSAGE % replacements
    )

    return template.loader.render_to_string(
        'github/login_button.html', context)



from .clients import GitHubClient
from .settings import SETUP_URL_NAME
from ..core import views


class GitHubDialogRedirect(views.OAuthDialogRedirectView):
    """
    View that handles the redirects for the GitHub authorization dialog.
    """
    client_class = GitHubClient


class GitHubCallback(views.OAuthCallbackView):
    """
    View that handles the GitHub OAuth callback.
    """
    client_class = GitHubClient


class GitHubSetup(views.OAuthSetupView):
    """
    View that handles the setup of a retrieved GitHub account.
    """
    client_class = GitHubClient
    setup_url = SETUP_URL_NAME


class GitHubOAuthDisconnect(views.OAuthDisconnectView):
    """
    View that handles the disconnect of a GitHub account.
    """
    client_class = GitHubClient



from django.core.exceptions import ImproperlyConfigured

from ..core.settings import CONFIGURATION, COOKIE_MAX_AGE


# Tries to get the Moves app configuration, if the configuration is not
# provided or is incorrect raises an ImproperlyConfigured exception.
MOVES_APP = CONFIGURATION.get('MOVES_APP', None)

if not MOVES_APP:
    raise ImproperlyConfigured(
        'You have socialnetworks.moves in your ISTALLED_APPS, but you do '
        'not specify any "MOVES_APP" settings inside '
        'SOCIALNETWORK_CONFIGURATION.'
    )

else:
    APP_ID = MOVES_APP.get('APP_ID', None)
    APP_SECRET = MOVES_APP.get('APP_SECRET', None)
    SCOPE = ','.join(MOVES_APP.get('SCOPE', ['activity']))
    SESSION_KEY = MOVES_APP.get('SESSION_KEY', 'dsnmv')
    SESSION_FIELDS = ','.join(MOVES_APP.get('SESSION_FIELDS', []))
    SETUP_URL_NAME = MOVES_APP.get('SETUP_URL_NAME', None)

    if not APP_ID:
        raise ImproperlyConfigured(
            'A proper "APP_ID" must be specified in order to use '
            'socialnetworks.moves module.'
        )

    if not APP_SECRET:
        raise ImproperlyConfigured(
            'A proper "APP_SECRET" must be specified in order to use '
            'socialnetworks.moves module.'
        )



import importlib

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


__all__ = ['EMAIL_IS_USERNAME', 'SETUP_FORM_CLASS', 'SETUP_TEMPLATE']

# Tries to get the module configuration, if the configuration is not
# provided or is incorrect raises an ImproperlyConfigured exception.
if [True for app in settings.INSTALLED_APPS if 'socialnetwork' in app]:
    CONFIGURATION = getattr(settings, 'SOCIALNETWORKS_CONFIGURATION', None)

    if not CONFIGURATION:
        raise ImproperlyConfigured(
            'You have one or more socialnetworks modules in your '
            'INSTALLED_APPS but you do not specify any '
            '"SOCIALNETWORKS_CONFIGURATION" in your settings.'
        )

    else:
        EMAIL_IS_USERNAME = CONFIGURATION.get('EMAIL_IS_USERNAME', True)
        COOKIE_MAX_AGE = CONFIGURATION.get('COOKIE_MAX_AGE', 900)
        SETUP_TEMPLATE = CONFIGURATION.get(
            'SETUP_TEMPLATE',
            'setup_form.html'
        )
        ACTIVATE_ALREADY_REGISTERED_USERS = CONFIGURATION.get(
            'ACTIVATE_ALREADY_REGISTERED_USERS',
            False
        )

        # Imports and defines the setup form class.
        form_class = CONFIGURATION.get(
            'SETUP_FORM_CLASS',
            'socialnetworks.core.forms.SocialUserCreationForm'
        )

        module_ = '.'.join(form_class.split('.')[:-1])
        class_ = form_class.split('.')[-1]
        module = importlib.import_module(module_)

        SETUP_FORM_CLASS = getattr(module, class_)



import uuid

from . import settings
from .models import LinkedInOAuthProfile
from ..core.clients import OAuth2Client


class LinkedInClient(OAuth2Client):
    """
    Client to connect to the LinkedIn REST API.
    """
    # Configuration
    service_name = 'LinkedIn'
    app_key = settings.APP_ID
    app_secret = settings.APP_SECRET
    model = LinkedInOAuthProfile

    uid_label = 'id'
    expiration_label = 'expires_in'

    service_api_url = 'https://api.linkedin.com/v1/'
    authorization_url = 'https://www.linkedin.com/uas/oauth2/authorization'
    access_token_url = 'https://www.linkedin.com/uas/oauth2/accessToken'
    token_debug_url = 'people/~:(id)'
    session_key = 'socialnetworks:linkedin'

    def compose_authorization_url(self, callback_url):
        """
        Return the url to request user authorization at LinkedIn.
        """
        params = {
            'client_id': self.app_key,
            'redirect_uri': callback_url,
            'response_type': 'code',
            'scope': self.scope,
            'state': uuid.uuid4(),
        }

        return self.encode_url(self.authorization_url, params)

    def get_auth_params(self, token=None):
        """
        Compose the authentication params to connect to the LinkedIn API.
        """
        # Pass the access token as a parameter for the LinkedIn OAuth2
        # service and tell to return the response in json format.
        if self.profile and not token:
            token = self.profile.oauth_access_token

        return {'format': 'json', 'oauth2_access_token': token}

    def get(self, api_endpoint, params={}, token=None):
        """
        Method to perform GET requests to the LinkedIn API.
        """
        params.update(self.get_auth_params(token))
        url = self.service_api_url + api_endpoint

        return self._get(url, params=params).json()

    def post(self, api_endpoint, data=None, params={}, token=None):
        """
        Method to perform POST requests to the LinkedIn API.
        """
        params.update(self.get_auth_params(token))
        url = self.service_api_url + api_endpoint

        return self._post(url, data=data, params=params).json()

    def debug_access_token(self, token=None):
        """
        Check if the given access token is yet valid.

        Return a tuple of two objects where the first element is a boolean
        that tells whether the token is valid or not and the second element is
        the data resulting of the token validation.
        """
        if self.profile and not token:
            token = self.profile.oauth_access_token

        r = self.get(self.token_debug_url, token=token)

        return (False if 'errorCode' in r else True, r)

    def retrieve_user_data(self):
        """
        Return the available data of the user from LinkedIn.
        """
        # Defines the fields to retrieve.
        fields = ','.join(['firstName', 'lastName', 'email-address'])

        # Fetches the user's data from the API.
        r = self.get('people/~:(%s)' % fields)

        return {
            'first_name': r['firstName'],
            'last_name': r['lastName'],
            'email': r['emailAddress'],
        }



from .clients import TwitterClient
from .settings import SETUP_URL_NAME
from ..core import views


class TwitterDialogRedirect(views.OAuthDialogRedirectView):
    """
    View that handles the redirects for the Twitter authorization dialog.
    """
    client_class = TwitterClient


class TwitterCallback(views.OAuthCallbackView):
    """
    View that handles the Twitter OAuth callback.
    """
    client_class = TwitterClient
    setup_url = SETUP_URL_NAME


class TwitterSetupView(views.OAuthSetupView):
    """
    View that handles the setup of a retrieved Twitter account.
    """
    client_class = TwitterClient


class TwitterOAuthDisconnect(views.OAuthDisconnectView):
    """
    View that handles the disconnect of a Twitter account.
    """
    client_class = TwitterClient



import uuid

from . import settings
from .models import GitHubOAuthProfile
from ..core.clients import OAuth2Client


class GitHubClient(OAuth2Client):
    """
    Client to connect to the GitHub REST API.
    """
    # Configuration
    service_name = 'GitHub'
    app_key = settings.APP_ID
    app_secret = settings.APP_SECRET
    app_access_token = settings.APP_ACCESS_TOKEN
    scope = settings.SCOPE
    model = GitHubOAuthProfile

    uid_label = 'id'
    expiration_label = 'expires'

    authorization_url = 'https://github.com/login/oauth/authorize'
    access_token_url = 'https://github.com/login/oauth/access_token'
    token_debug_url = 'user'
    service_api_url = 'https://api.github.com/'
    session_key = 'socialnetworks:github'

    def compose_authorization_url(self, callback_url):
        """
        Return the url to request user authorization at GitHub.
        """
        params = {
            'client_id': self.app_key,
            'redirect_uri': callback_url,
            'scope': self.scope,
            'state': uuid.uuid4()
        }

        return self.encode_url(self.authorization_url, params)

    def debug_access_token(self, token=None):
        """
        Check if the given access token is yet valid.

        Return a tuple of two objects where the first element is a boolean
        that tells whether the token is valid or not and the second element is
        the data resulting of the token validation.
        """
        if self.profile and not token:
            token = self.profile.oauth_access_token

        r = self.get(self.token_debug_url, auth_params={'access_token': token})

        return (False if 'message' in r else True, r)

    def retrieve_user_data(self):
        """
        Return the available data of the user from GitHub.
        """
        r = self.get('user')

        # Parses the "name" of GitHub, simply splits the name by white
        # spaces, the first name is the first element of the resulting list,
        # the last name are the remaining elements joined again by white
        # spaces. If there are not white spaces in the name, then the first
        # name is the name returned by GitHub and the last name is
        # left blak.
        name = r['name']
        first_name = name.split(' ')[0] if ' ' in name else name
        last_name = ' '.join(name.split(' ')[1:]) if ' ' in name else None

        return {
            'username': r['login'],
            'first_name': first_name,
            'last_name': last_name,
            'email': r['email']
        }



from . import settings
from .models import TwitterOAuthProfile
from ..core.clients import OAuth1Client


class TwitterClient(OAuth1Client):
    """
    Client to connect to the Twitter REST API.
    """
    # Configuration
    service_name = 'Twitter'
    app_key = settings.APP_ID
    app_secret = settings.APP_SECRET
    model = TwitterOAuthProfile

    service_api_url = 'https://api.twitter.com/1.1/'
    request_token_url = 'https://api.twitter.com/oauth/request_token'
    authorization_url = 'https://api.twitter.com/oauth/authorize'
    access_token_url = 'https://api.twitter.com/oauth/access_token'
    token_debug_url = 'account/verify_credentials.json'
    session_key = 'socialnetworks:twitter'

    def compose_authorization_url(self, token):
        """
        Return the url to request user authorization at Twitter.
        """
        params = {'oauth_token': token}

        return self.encode_url(self.authorization_url, params)

    def debug_access_token(self):
        """
        Check if the given access token is yet valid.

        Return a tuple of two objects where the first element is a boolean
        that tells whether the token is valid or not and the second element is
        the data resulting of the token validation.
        """
        if self.profile:
            r = self.get(self.token_debug_url)

            if 'errors' in r:
                return (False, r)
            else:
                return (True, r)

    def retrieve_user_data(self):
        """
        Return the available data of the user from Twitter.
        """
        r = self.get(self.token_debug_url)

        # Parses the "name" of Twitter, simply splits the name by white
        # spaces, the first name is the first element of the resulting list,
        # the last name are the remaining elements joined again by white
        # spaces. If there are not white spaces in the name, then the first
        # name is the name returned by Twitter and the last name is
        # left blak.
        name = r['name']
        first_name = name.split(' ')[0] if ' ' in name else name
        last_name = ' '.join(name.split(' ')[1:]) if ' ' in name else None

        return {
            'username': r['screen_name'],
            'first_name': first_name,
            'last_name': last_name,
        }



from .models import MovesAppOAuthProfile
from ..core.backends import BaseSocialBackend


class MovesAppBackend(BaseSocialBackend):
    """
    Backend to handle login with Moves app.
    """
    model = MovesAppOAuthProfile



from .clients import FacebookClient
from .settings import SETUP_URL_NAME
from ..core import views


class FacebookDialogRedirect(views.OAuthDialogRedirectView):
    """
    View that handles the redirects for the Facebook authorization dialog.
    """
    client_class = FacebookClient


class FacebookCallback(views.OAuthCallbackView):
    """
    View that handles the Facebook OAuth callback.
    """
    client_class = FacebookClient


class FacebookSetup(views.OAuthSetupView):
    """
    View that handles the setup of a retrieved Facebook account.
    """
    client_class = FacebookClient
    setup_url = SETUP_URL_NAME


class FacebookOAuthDisconnect(views.OAuthDisconnectView):
    """
    View that handles the disconnect of a Facebook account.
    """
    client_class = FacebookClient



import json
import requests

from requests_oauthlib import OAuth1, OAuth2
from urlparse import parse_qsl

from django.contrib.auth import authenticate, login


class BaseOAuthClient(object):
    """
    Base class that defines the client for OAuth services.
    """
    # The version of the OAuth protocol used by the client.
    oauth_version = None

    # The name of the service.
    service_name = None

    # The app's client id provided by the service.
    app_key = None

    # The app's secret provided by the service.
    app_secret = None

    # The app's access token provided by the service.
    app_access_token = None

    # The base url for make requests to the service API.
    service_api_url = None

    # The url where the service give us the request token.
    request_token_url = None

    # The url where the service shows the auth/login dialog to the user.
    authorization_url = None

    # The url where the service give us the access token.
    access_token_url = None

    # The label of the request token in the service.
    request_token_label = None

    # The label of the request token secret in the service.
    request_token_secret_label = None

    # The label of the access token in the service.
    access_token_label = None

    # The label of the access token secret in the service.
    access_token_secret_label = None

    # The label of the refresh token in the service if available.
    refresh_token_label = None

    # The label of the user's id in the service.
    uid_label = None

    # The label of the access token expiration in the service.
    expiration_label = None

    # The url where the access token can be debbuged.
    token_debug_url = None

    # The key for the session dictionary where the flow data will be stored.
    session_key = None

    # The label that is used by the service to name its OAuth verifier.
    verifier_label = 'None'

    # The permissions that the app will ask for.
    scope = None

    # The model where the profiles are stored.
    model = None

    def __init__(self, profile=None):
        self.profile = profile if profile else None

    def _get(self, url, **kwargs):
        """
        Base method to perform GET requests by wrapping the
        'requests' python library.
        """
        return requests.get(url, **kwargs)

    def _post(self, url, data=None, **kwargs):
        """
        Base method to perform POST requests by wrapping the
        'requests' python library.
        """
        return requests.post(url, data=data, **kwargs)

    def encode_url(self, url, params={}):
        """
        Returns the encoded url with thw given parameters.
        """
        try:
            r = requests.Request(url=url, params=params).prepare()
            return r.url

        except:
            encoder = requests.models.RequestEncodingMixin()
            enc_params = encoder._encode_params(params)
            return url + '?' + enc_params

    def login(self, request, uid):
        """
        Logs the user in.
        """
        login(request, authenticate(**{'service_uid': uid}))

    def parse_response(self, data):
        """
        Returns a dictionary of the parsed response.
        """
        try:
            return json.loads(data)

        except:
            return dict(parse_qsl(data))

    def get_auth_params(self):
        """
        Returns a dictionary containing the proper parameters to compose
        the OAuth auth header.
        """
        if self.profile:
            if self.oauth_version == 1:
                return {
                    'resource_owner_key': self.profile.oauth_access_token,
                    'resource_owner_secret': (
                        self.profile.oauth_access_token_secret),
                }

            elif self.oauth_version == 2:
                return {
                    'access_token': self.profile.oauth_access_token,
                    'token_type': 'bearer'
                }

        else:
            raise NameError('\'profile\' is not defined.')

    def compose_auth(self, auth_args={}):
        """
        Returns the auth header for the OAuth requests.

        Subclasses must implement this method.
        """
        raise NotImplementedError

    def compose_authorization_url(self, callback_url):
        """
        Return the url where the user should be redirected to request
        authorization for its account details.

        Subclasses must implement this method.
        """
        raise NotImplementedError

    def get_request_token(self):
        """
        Connects with the service to obtain the OAuth request token and
        returns a dictionary with the parsed token.

        OAuth1 subclasses must implement this method.
        """
        raise NotImplementedError

    def get_access_token(self):
        """
        Connects with the service to obtain the OAuth access token and
        returns a dictionary with the parsed token.

        Subclasses must implement this method.
        """
        raise NotImplementedError

    def debug_access_token(self):
        """
        Connects with the service to check if the access token is valid and
        returns a tuple where the first element is a boolean indicating the
        validity and the second is the response from the service.

        Subclasses must implement this method.
        """
        raise NotImplementedError

    def refresh_access_token(self, profile=None, update=True):
        """
        Connects with the service to request a new access token for the
        given profile.

        Pass update=True to force the given profile to be updated with the
        retrieved data.

        Subclasses must implement this method.
        """
        raise NotImplementedError

    def retrieve_user_data(self, profile=None):
        """
        Return the available user data that can be retrieved from the
        service's API in a python/django friendly format.

        Subclasses must implement this method.
        """
        raise NotImplementedError

    def get(self, api_endpoint, params={}, auth_params=None, raw=False):
        """
        Makes a get request to the service's API and returns the response
        in json format.

        Parameters:
            - api_endpoint: a string that defines the endpoint where the
                request will be directed, this will be concatenated with
                the base API url.
            - params: a dictionary containing all the extra parameters that
                will be passed to the API.
            - auth_params: a dictionary containing all the extra parameters
                needed to compose the OAuth auth header.
        """
        if self.profile and not auth_params:
            auth_params = self.get_auth_params()

        url = self.service_api_url + api_endpoint

        response = self._get(
            url, params=params,
            auth=self.compose_auth(auth_params)
        )

        return raw and response or response.json()

    def post(self,
             api_endpoint, data=None, params={}, auth_params=None, raw=False):
        """
        Makes a post request to the service's API and returns the response
        in json format.

        Parameters:
            - api_endpoint: a string that defines the endpoint where the
                request will be directed, this will be concatenated with
                the base API url.
            - data: the data that will be sent to the API in the POST body
                of the request.
            - params: a dictionary containing all the extra parameters that
                will be passed to the API.
            - auth_params: a dictionary containing all the extra parameters
                needed to compose the OAuth auth header.
        """
        if self.profile and not auth_params:
            auth_params = self.get_auth_params()

        url = self.service_api_url + api_endpoint

        response = self._post(
            url, data=data, params=params,
            auth=self.compose_auth(auth_params)
        )

        return raw and response or response.json()


class OAuth1Client(BaseOAuthClient):
    """
    Base client for OAuth1 services.
    """
    oauth_version = 1
    verifier_label = 'oauth_verifier'
    request_token_label = 'oauth_token'
    request_token_secret_label = 'oauth_token_secret'
    access_token_label = 'oauth_token'
    access_token_secret_label = 'oauth_token_secret'
    uid_label = 'user_id'

    def compose_auth(self, auth_params={}):
        auth_params.update({'client_secret': self.app_secret})

        return OAuth1(self.app_key, **auth_params)

    def get_request_token(self, callback=None):
        # Gets the authentication header.
        oauth = self.compose_auth()

        # Requesting and parsing the request token.
        r = self._post(
            self.request_token_url, auth=oauth,
            params={'oauth_callback': callback}
        )

        return self.parse_response(r.content)

    def get_access_token(self, request_token=None,
                         request_token_secret=None, verifier=None):
        # Gets the authentication header.
        oauth = self.compose_auth({
            'resource_owner_key': request_token,
            'resource_owner_secret': request_token_secret,
            'verifier': verifier
        })

        # Requesting and parsing the access token.
        r = self._post(self.access_token_url, auth=oauth)

        return self.parse_response(r.content)


class OAuth2Client(BaseOAuthClient):
    """
    Base client for OAuth2 services.
    """
    oauth_version = 2
    verifier_label = 'code'
    access_token_label = 'access_token'
    refresh_token_label = 'refresh_token'
    uid_label = 'user_id'

    def compose_auth(self, auth_params={}):
        return OAuth2(client_id=self.app_key, token=auth_params)

    def get_access_token(self, verifier=None, callback=None):
        # Composes the parameters for the request.
        params = {
            'grant_type': 'authorization_code',
            'client_id': self.app_key,
            'client_secret': self.app_secret,
            'redirect_uri': callback,
            self.verifier_label: verifier
        }

        # Requesting and parsing the access token.
        r = self._post(self.access_token_url, params=params)

        return self.parse_response(r.content)



from .models import TwitterOAuthProfile
from ..core.backends import BaseSocialBackend


class TwitterBackend(BaseSocialBackend):
    """
    Backend to handle login with Twitter.
    """
    model = TwitterOAuthProfile
